class Array:
    def __init__(self, arg1, zero=None):
        if type(arg1) is list:
            self.size = len(arg1)
            self.data = arg1[:]

        elif type(arg1) is int:
            self.size = arg1
            self.data = self.size * [zero]

    def __str__(self):
        return str(self.data)

    def __repr__(self):
        return str(self)

    def check_index(self, index):
        assert 0 <= index < len(self.data), "array index out of range"

    def __setitem__(self, index, value):
        self.check_index(index)
        self.data[index] = value

    def __getitem__(self, index):
        self.check_index(index)
        return self.data[index]

    def __len__(self):
        return len(self.data)

    def __eq__(self, outro):
        if type(outro) is list:
            return self.data == outro
        return outro.data == self.data



from builtins import input
from uuid import getnode as get_mac

import sys
import os
import webbrowser
import logging
import pkg_resources

from requests import ConnectionError

import p1
from p1.colors import *
from p1.utils import cprint, data2json, _assert
from p1.jsonfile import JsonFile

def main():
    try:
        sitename = sys.argv[2] if len(sys.argv) > 2 else '_DEFAULT'
        login(sitename)
    except Exception as e:
        cprint(LRED, "ops!")
        raise e


def login(sitename):
    """login in site"""

    # fetch site urls
    site = p1.get_site(sitename)
    _assert(site is not None, "Site %s not found in config.yaml" % sitename)

    # check site login style
    api_login_url = site.api_login_url()
    _assert(api_login_url, "Site %s has no login url" % site.name)

    # perform cli tools login; fetch auth and acces urls and login token
    p1_version = pkg_resources.get_distribution('p1ufcg').version
    response = site.post(api_login_url, {"mac": str(get_mac()), "p1_version": p1_version})
    response.encoding = 'utf-8'
    user_auth_url = response.json()['user-auth-url']
    api_access_url = response.json()['api-access-url']

    # open browser at authorization page
    cprint(YELLOW, user_auth_url)
    cprint(LCYAN, f"Abrir o browser para authorizar o login? (S/n) ", end="")
    if input() in "Ss":
        if webbrowser.open(user_auth_url):
            print(f"Aguardando autorização via browser…")
        else:
            cprint(LRED, "Ops! Não foi possível abrir o browser.")
            cprint(WHITE, "Visite o link acima para autorizar o login.")
    else:
        cprint(WHITE, "Visite o link acima para autorizar o login.")

    # request access code
    for i in range(10):
        response = site.get(f'{api_access_url}')
        if response.status_code == 500:
            cprint(LRED, "O login falhou (timeout)")
            cprint(LGREEN, "Tente novamente...")
            #sys.exit(1)
            continue
        authorization = response.json()
        if response.status_code == 200: break

    # check authorization
    if 'authorized' not in authorization:
        cprint(LRED, 'Login não autorizado')
        logging.info('Login não autorizado')
        sys.exit(1)

    # check whether the login worked
    if not 'tst_token' in authorization:
        cprint(LRED, 'O login falhou (não autorizado)')
        logging.info('tst_token não encontrado na resposta')
        return

    # save token
    tokens = JsonFile(os.path.expanduser('~/.p1/tokens.json'))
    tokens[site.name] = authorization['tst_token']
    tokens.writable = True
    tokens.save()

    msg = f"Você está logado no site de p1 como {YELLOW}{authorization['email']}{RESET}"
    cprint(LGREEN, msg)
    logging.info(msg)



from .utils import _assert, cprint
from .colors import *

from .p1 import coverit
from .p1 import get_config
from .p1 import dirtype
from .p1 import get_site

from .array import Array
from .p1 import P1STYLE




from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(name='p1ufcg',
      version="0.3.6",
      description='P1 UFCG',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='http://github.com/daltonserey/p1',
      author='Dalton Serey',
      author_email='daltonserey@gmail.com',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      scripts=[],
      python_requires='>=3.6',
      install_requires=[
        'pyyaml>=5.4.1',
        'requests>=2.6.1',
        'cachecontrol[filecache]',
        'tst>=0.17.0',
        'pytest-tst>=0.1.2',
        'questionary',
        'pudb'
      ],
      entry_points = {
        'console_scripts': [
            'p1=p1.commands:main',
        ]
      },
      zip_safe=False)



import sys
import os
import re
from pathlib import Path

import questionary
from questionary import print as qprint

import p1
from p1.colors import *
from p1.utils import cprint
from p1.utils import _assert
from p1.utils import data2json
from p1.jsonfile import JsonFile
from p1 import P1STYLE

BASEDIR = Path(os.environ.get("P1_DIR") or p1.get_config().get("basedir", "~/p1")).expanduser()

# resquício dos comandos do tst
site = p1.get_site("prog1")

def validate_use(args):
    option_args = [o for o in args if o.startswith("--")]
    try:
        assert all(o in {"--overwrite", "--dir", "-o", "-d"} for o in option_args)
    except AssertionError as e:
        return False

    return True


def unidade(u):
    try:
        numero = re.search("\d+|$", u).group()
        if numero.isdigit():
            return f"u{numero}"

    except TypeError as e:
        # unit é None
        pass

    return "ex"


def main():
    # separa argumentos
    args = sys.argv[2:]
    _assert(validate_use(args), syntax_help())

    # determina especificação da atividade a buscar
    spec = next((a for a in args if not a.startswith("--")), "@")

    # opção --overwrite?
    overwrite = any(a == "--overwrite" for a in args)

    if os.path.isdir(spec):
        # "p1 checkout <DIR>": atualização do checkout em <DIR>
        p1_filename = f"{args[0]}/.p1/assignment.json"
        _assert(os.path.exists(p1_filename), f"Diretório {args[0]} não contém atividade de p1")
        p1file = JsonFile(p1_filename)
        activity_dir = args[0]
        email, key = p1file["user"], p1file["iid"]

    else:
        # ou é <pattern>[@<email>]
        # ou é um IID
        # ou foi invocação sem argumentos (nesse caso spec será '@')
        pattern, email = spec.split("@") if "@" in spec else (spec, "")
        questionary.print("Identificando atividades no servidor", "bold green")
        response = site.get(f"/available?email={email}")
        if response.status_code != 200:
            data = response.json()
            print(data["messages"][0])
            sys.exit(1)

        available = response.json()["available"]
        filtered = [act for act in available if pattern.lower() in act["label"].lower()]
        if not filtered and pattern.isdigit():
            activity_iid = int(pattern)
            filtered = [act for act in available if act['iid'] == activity_iid]
        _assert(filtered, f"nenhuma atividade encontrada: {pattern}")

        opcoes = []
        for (i,a) in enumerate(filtered):
           unit = unidade(a.get("unit"))
           opcoes.append(questionary.Choice(f'{unit} / {a["dirname"]}: {a["label"]}', value=i))

        idx_opcao = questionary.select(
            "Atividades disponíveis: ",
            opcoes,
            style=P1STYLE,
            instruction="(use as setas para escolher)",
            use_jk_keys=True,
        ).ask()
        iid = str(filtered[idx_opcao]["iid"])
        activity_dir = str(BASEDIR / filtered[idx_opcao]["dirname"])

    assignments = Path('~/p1').expanduser().rglob('.p1/assignment.json')
    assignments = [p for p in assignments]
    iguais = [p for p in assignments if str(JsonFile(p)['iid']) == iid]
    if len(iguais) > 1:
        qprint('Erro. Há vários diretórios com a mesma atividade:', style="bold red")
        for a in iguais:
            directory = str(a.parent.parent).replace(str(Path.home()), "~")
            qprint(directory, style="#bbbbbb")
        qprint("Apague os diretórios desnecessários antes de prosseguir.", style="bold #ff9d00")
        qprint("Checkout abortado.", style="bold #ff9d00")
        sys.exit(1)
        
    if len(iguais) == 1 and iguais[0].parent.parent != activity_dir:
        iguais[0].parent.parent.rename(activity_dir)

    checkout(iid, activity_dir, overwrite)


def syntax_help():
    return ("Usage: p1 checkout <key>[@<site>] [<directory>]\n"
           "       p1 checkout <url> [<directory>]\n\n"
           "   or: p1 checkout\n"
           "       (inside a p1 directory)")


def save_file(filename, content, mode):
    def octal_mode(mode):
        return {
            (False , False): 0o444,
            (False ,  True): 0o555,
            (True  , False): 0o644,
            (True  ,  True): 0o755
        }["w" in mode, "x" in mode]

    subdirs = os.path.dirname(filename)
    if not os.path.isdir(subdirs):
        os.makedirs(subdirs)

    with open(filename, encoding="utf-8", mode="w") as f:
        f.write(content)

    os.chmod(filename, octal_mode(mode))


def save_selected_files(savetable):
    for i in range(len(savetable)):
        line = savetable[i]
        if line[2] == 'unchanged':
            savetable[i][3] = 'skipped'
            continue

        try:
            filename = line[1]
            if os.path.exists(filename):
                os.chmod(filename, 0o644)

            mode = line[0].get('mode', '644')
            save_file(filename, line[0]['content'], mode)
            savetable[i][3] = 'saved'

        except (IOError, OSError) as e:
            savetable[i][3] = 'failed'
            assert False, f"CRITICAL ERROR: failed saving file '{line[1]}'"


def get_save_table(files, basedir):
    """
    Return a table that helps saving files to FS.
    Each line of the table has 4 columns:
    - the file itself
    - the filename to be saved to
    - the situation wrt to current FS: notfound, unchanged, changed
    - an empty cell to write the final status after saving
    """
    savetable = []
    for f in files:
        save_name = "%s/%s" % (basedir, f['name'])
        if not os.path.exists(save_name):
            savetable.append([f, save_name, 'notfound', None])
        else:
            # a version of the file already exists
            old_contents = open(save_name, encoding='utf-8').read()
            new_contents = f['content']
            if old_contents == new_contents:
                savetable.append([f, save_name, 'unchanged', None])
            else:
                savetable.append([f, save_name, 'changed', None])

    return savetable


def checkout(iid, destdir, overwrite_allowed):
    """checkout activity/assignment from site/collection"""

    # fetch activity
    cprint(LGREEN, "Buscando atividade no servidor (%s)" % iid)
    activity, response = site.get_activity(iid)
    if response.status_code == 404:
        activity, response = site.get_directory(iid)

    if response.status_code == 401:
        cprint(LRED, 'Checkout não autorizado (%s)' % site.last_response.status_code)
        cprint(WHITE, 'Execute: p1 login')
        return

    elif response.status_code == 404:
        cprint(LRED, 'Atividade não encontrada (%s)' % site.last_response.status_code)
        return

    elif response.status_code in [400, 412]:
        cprint(LRED, 'Checkout não permitido (%s)' % site.last_response.status_code)
        cprint(LRED, 'Problema: %s' % site.last_response.json()['messages'][0])
        return

    _assert(activity, "Não foi possível fazer o checkout (%s)" % response.status_code)

    # analyze what must be saved to FS
    savetable = get_save_table(activity['files'], destdir)
    num_to_overwrite = 0
    for line in savetable:
        if line[2] == 'changed':
            num_to_overwrite += 1
            if not overwrite_allowed: cprint(LRED, f"{line[1]}")

    if num_to_overwrite and not overwrite_allowed:
        cprint(YELLOW, "Estes arquivos serão alterados! Confirma? (s/N)? ", end="")
        if input() != "s":
            cprint(YELLOW, 'Checkout cancelado pelo usuário')
            sys.exit(1)

    # save 'notfound' and 'changed' files
    save_selected_files(savetable)
    num_saved = 0
    for line in savetable:
        filename = line[1].replace(str(Path.home()), "~")
        if line[3] == 'skipped':
            cprint(RESET, f"ignorando: {filename}")
        elif line[3] == 'saved':
            cprint(LCYAN, f" salvando: {filename}")
            num_saved += 1

    cprint(LGREEN, f"{num_saved} arquivo(s) gravados")

    content = data2json({
            "site": site.name,
            "key": iid,
            "iid": activity.get('iid'),
            "user": activity.get('user'),
            "dirname": activity.get('dirname'),
            "full_resource": activity['_response'].json() if '_response' in activity else None
        })
    save_file(f'{destdir}/.p1/assignment.json', content, "rw")



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import p1
import sys

from p1.colors import *
from p1.utils import cprint

def main():
    print("comando info ainda não implementado", file=sys.stderr)
